# Generated by Django 4.1.7 on 2023-08-16 05:25

from django.db import migrations, models
import uuid


class Migration(migrations.Migration):

    dependencies = [
        ('nets_core', '0004_remove_userdevice_device_id_userdevice_uuid'),
    ]

    operations = [
        migrations.AddField(
            model_name='userdevice',
            name='uuid',
            field=models.UUIDField(default=uuid.uuid4, editable=False, unique=True, verbose_name='UUID'),
        ),
    ]
