{% extends 'nets_core/email/base_email.html' %}
{% load i18n %}

{% block title %}
    {% trans 'Here is your verification code' %}
{% endblock title %}

{% block content %}
    {% trans 'Use this code where is required. If you do not request this, please report this to us.' %}
    {{ code }}
{% endblock content %}